#ifndef SHELL_LOCAL_H
#define SHELL_LOCAL_H

#define MAX_LIST_SIZE 256
#define MAX_STACK_DEPTH 64

typedef struct Shell_Function
{
	Hash_Data h;
	// do NOT change the order - it has to be right after h!
	const char *description;
	Shell_Function_CB callback;
}
Shell_Function;

typedef struct Shell_Parser {
	NUint line;
	const char *currentChar;
	char *token;
}
Shell_Parser;

typedef struct Shell_Statement {
	// number of items in list
	NUint listSize;
	// items of the current statement/list
	char *list[MAX_LIST_SIZE];
} Shell_Statement;

typedef struct Shell_Block {
	struct Shell_Block *parent;
	// this is allocated by the current stack level and should be freed by the caller
	char *returnText;
	// preinitialized parser
	Shell_Parser *parser;
	// current statement
	Shell_Statement *currentStatement;
} 
Shell_Block;

typedef struct ShellState
{
	// whether Command system is active
	Nbool active;
	// script still waiting to be executed
	char *commandText;
	// hash lookup for command/alias/cvar names
	Hash *functionHash;
	// current stack depth
	NUint executionDepth;
    // used by ArgC and ArgV, etc
	Shell_Block *currentBlock;
	// memory zone for commands
	Mem_Zone *zone;
	Mem_Zone *interpreterZone;
}
ShellState;

extern ShellState Shell;

// executes the current execution frame
//void Shell_Execute( void );

Shell_Function * Shell_FindFunction( const char *name );

void Shell_Interpreter_InterpretSubBlock( Mem_Zone *zone, char **returnOut );
void Shell_Interpreter_InterpretTextBlock(const char *text, Mem_Zone *zone, char **returnOut );

#endif